#include <stdint.h>
#include <assert.h>

#include "../compat/pgmspace.h"
#include "bitmapscroller.h"
#include "laborlogo.h"


static uint8_t laborlogo_getChunk(unsigned char const nBitPlane,
                                  unsigned char const nChunkX,
                                  unsigned char const nChunkY,
                                  unsigned int const nFrame)
{
	assert(nChunkX < 6);
	assert(nChunkY < 48);

	static uint8_t aBitmap[48][6] PROGMEM =
		{{0xFF, 0xFF, 0xE0, 0x07, 0xFF, 0xFF},
		 {0xFF, 0xFF, 0x00, 0xF8, 0xFF, 0xFF},
		 {0xFF, 0xF8, 0x00, 0xFF, 0x1F, 0xFF},
		 {0xFF, 0xF0, 0x00, 0xFF, 0xEF, 0xFF},
		 {0xFF, 0xC0, 0x00, 0xFF, 0xF3, 0xFF},
		 {0xFF, 0x80, 0x00, 0xFF, 0xFD, 0xFF},
		 {0xFF, 0x00, 0x00, 0xFF, 0xFE, 0xFF},
		 {0xFE, 0x00, 0x03, 0x3F, 0xFF, 0x7F},
		 {0xFC, 0x00, 0x04, 0xDF, 0xFF, 0xBF},
		 {0xF8, 0x00, 0x08, 0xEF, 0xFF, 0xDF},
		 {0xF0, 0x00, 0x10, 0xF7, 0xFF, 0xEF},
		 {0xF0, 0x00, 0x10, 0xF7, 0xFF, 0xEF},
		 {0xE0, 0xC0, 0x10, 0xF7, 0xFF, 0xF7},
		 {0xC0, 0xC0, 0x10, 0xF7, 0xFF, 0xFB},
		 {0xC0, 0x40, 0x08, 0xEF, 0xFF, 0xFB},
		 {0xC0, 0x40, 0x04, 0xDF, 0xFF, 0xFB},
		 {0x80, 0x40, 0x03, 0x3F, 0xFF, 0xFD},
		 {0x87, 0xFC, 0x00, 0xFF, 0xFF, 0xFD},
		 {0x84, 0x0C, 0x00, 0xFF, 0xFF, 0xFD},
		 {0x04, 0x00, 0x1F, 0x07, 0xFF, 0xFE},
		 {0x04, 0x00, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x04, 0x00, 0x10, 0xF7, 0xFF, 0xFE},
		 {0xFF, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x00, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x00, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x00, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x00, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x00, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0x00, 0x80, 0x10, 0xF7, 0xFF, 0xFE},
		 {0xBF, 0xFC, 0x10, 0xF7, 0xFF, 0xFD},
		 {0xB0, 0x0C, 0x10, 0xF7, 0xFF, 0xFD},
		 {0x80, 0x00, 0x10, 0xF7, 0xFF, 0xFD},
		 {0xC0, 0x00, 0x10, 0xF7, 0xFF, 0xFB},
		 {0xC0, 0x00, 0x10, 0xF7, 0xFF, 0xFB},
		 {0xC0, 0x00, 0x10, 0xF7, 0xFF, 0xFB},
		 {0xE0, 0x00, 0x10, 0xF7, 0xFF, 0xF7},
		 {0xF0, 0x00, 0x10, 0xF7, 0xFF, 0xEF},
		 {0xF0, 0x00, 0x10, 0xF7, 0xFF, 0xEF},
		 {0xF8, 0x00, 0x10, 0xF7, 0xFF, 0xDF},
		 {0xFC, 0x00, 0x10, 0xF7, 0xFF, 0xBF},
		 {0xFE, 0x00, 0x1F, 0x07, 0xFF, 0x7F},
		 {0xFF, 0x00, 0x00, 0xFF, 0xFE, 0xFF},
		 {0xFF, 0x80, 0x00, 0xFF, 0xFD, 0xFF},
		 {0xFF, 0xC0, 0x00, 0xFF, 0xF3, 0xFF},
		 {0xFF, 0xF0, 0x00, 0xFF, 0xEF, 0xFF},
		 {0xFF, 0xF8, 0x00, 0xFF, 0x1F, 0xFF},
		 {0xFF, 0xFF, 0x00, 0xF8, 0xFF, 0xFF},
		 {0xFF, 0xFF, 0xE0, 0x07, 0xFF, 0xFF}};

	return pgm_read_byte(&aBitmap[nChunkY][nChunkX]);
}

void laborlogo()
{
	bitmap_scroll(48, 48, 1, 400, 75, laborlogo_getChunk);
}
